﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Drawing;
using System.Runtime;
using System.Xml;
using ScriptPortal.Vegas;

namespace Test_Script
{
    public class Class1
    {
        public Vegas myVegas;
        float threshold = 0.1f;
        DateTime myTime = DateTime.Now;

        public void Main(Vegas vegas)
        {
            myVegas = vegas;



            foreach (Track track in myVegas.Project.Tracks)
            {
                if (track.IsVideo())
                {
                    Timecode minStart = new Timecode(0);
                    foreach (TrackEvent evnt in track.Events)
                    {
                        if (evnt.Selected && evnt.Start >= minStart)
                        {
                            minStart = CheckImagesRegions(evnt);
                        }
                    }

                    RemoveGaps(track);
                }
            }
        }

        private Timecode CheckImagesRegions(TrackEvent evnt)
        {
            Timecode startFrame = evnt.Start;
            long FramesToProcess = evnt.Length.FrameCount;
            string FileLeft = myVegas.TemporaryFilesPath + Path.DirectorySeparatorChar + "0.png";
            string FileRight = "";
            myVegas.SaveSnapshot(FileLeft, ImageFileFormat.PNG, startFrame);
            Bitmap img1 = new Bitmap(FileLeft);
            Bitmap img2 = null;
            string Result = myTime.ToString() + "\r\n";
            bool first = true;
            bool PreviousWasBad = false;
            Timecode regionStart = evnt.Start;

            for (int CurrFrame = 1; CurrFrame < FramesToProcess; CurrFrame++)
            {
                FileRight = myVegas.TemporaryFilesPath + Path.DirectorySeparatorChar + CurrFrame.ToString() + ".png";
                Timecode CurrPos = startFrame + Timecode.FromFrames(CurrFrame);
                myVegas.SaveSnapshot(FileRight, ImageFileFormat.PNG, (startFrame + Timecode.FromFrames(CurrFrame)));
                img2 = new Bitmap(FileRight);

                float percentDifferent = CompareImagesPercent(img1, img2);


                string s = "CurrFrame = " + CurrFrame + "   %= " + percentDifferent;
                if (percentDifferent < threshold)
                {
                    if (!PreviousWasBad)
                    {
                        regionStart = CurrPos - Timecode.FromFrames(1);
                        first = false;
                        s += "  regionStart = " + regionStart.ToString() + "   and first = false";

                    }
                    PreviousWasBad = true;
                    s += "    PreviousWasBad now set to true - CurrFrame = " + CurrFrame + "  event length = " + evnt.Length.FrameCount;

                    if (CurrFrame == (FramesToProcess - 1))
                    {
                        evnt = RemoveRegion(evnt, regionStart, CurrPos - regionStart + Timecode.FromFrames(1));

                        PreviousWasBad = false;
                        s += "   Region removed";
                    }

                }
                else
                {
                    if ((!first) && (PreviousWasBad || (CurrFrame == (FramesToProcess - 1))))
                    {
                        Timecode endRegion = CurrPos - regionStart - Timecode.FromFrames(1);
                        if (CurrFrame == (FramesToProcess - 1))
                        {
                            endRegion = endRegion + Timecode.FromFrames(2);
                        }
                        evnt = RemoveRegion(evnt, regionStart, endRegion);

                        PreviousWasBad = false;
                        s += "   Region removed";
                    }

                }

                s += "\r\n";
                Result += s;

                img1.Dispose();
                File.Delete(FileLeft);
                FileLeft = FileRight;
                img1 = img2;
            }

            img2.Dispose();
            File.Delete(FileRight);
            myTime = DateTime.Now;
            Result += myTime.ToString();

            Clipboard.SetText(Result);
            return evnt.Start + Timecode.FromFrames(1);
        }

        private TrackEvent RemoveRegion(TrackEvent evnt, Timecode regStart, Timecode regLength)
        {
            if (regStart != evnt.Start)
            {
                Timecode splitPoint = regStart - evnt.Start;
                TrackEvent rightEvent = evnt.Split(splitPoint);
                evnt = rightEvent;
            }
            if (evnt.Length == regLength)
            {
                evnt.Track.Events.Remove(evnt);
                evnt = null;
            }
            else
            {
                Timecode splitPoint = regStart - evnt.Start + regLength;
                TrackEvent rightEvent = evnt.Split(splitPoint);
                evnt.Track.Events.Remove(evnt);
                evnt = rightEvent;
            }

            return evnt;
        }

        private void RemoveGaps(Track track)
        {
            bool FirstSelectedFound = false;
            Timecode CurrentEnd = new Timecode(0);
            foreach (TrackEvent evnt in track.Events)
            {
                if (evnt.Selected)
                {
                    if (!FirstSelectedFound)
                    {
                        FirstSelectedFound = true;
                        CurrentEnd = evnt.End;
                    }
                    else
                    {
                        evnt.Start = CurrentEnd;
                        CurrentEnd = evnt.End;
                    }
                }
            }
        }

        private float CompareImagesPercent(Bitmap img1, Bitmap img2)
        {
            float diff = 0;

            for (int y = 0; y < img1.Height; y++)
            {
                for (int x = 0; x < img1.Width; x++)
                {
                    Color pixel1 = img1.GetPixel(x, y);
                    Color pixel2 = img2.GetPixel(x, y);

                    diff += Math.Abs(pixel1.R - pixel2.R);
                    diff += Math.Abs(pixel1.G - pixel2.G);
                    diff += Math.Abs(pixel1.B - pixel2.B);
                }
            }
            return 100 * (diff / 255) / (img1.Width * img1.Height * 3);
        }
    }
}

public class EntryPoint
{
    public void FromVegas(Vegas vegas)
    //public void FromVegas(Vegas vegas, String scriptFile, XmlDocument scriptSettings, ScriptArgs args)
    {
        Test_Script.Class1 test = new Test_Script.Class1();
        test.Main(vegas);
    }
}


